#!/usr/bin/python
import cgi
import commands
import os
import string
import time
import popen2
import random
from nodes import *

# Global definitions
RRD_ROOT_DIR = "/home/cafmon/batchmon/RRD"
RRDTOOL_BIN  = "/home/cafmon/bin/rrdtool"

colors = {
    1:"ff0000",2:"0000ff",3:"00ff00",5:"FFFF00",4:"00DDDD",6:"FF00FF",
    7:"af7817",8:"7931df",9:"980517",10:"578693",11:"f76541",12:"f5f3d7",
    13:"e18b6b",14:"7d1b7e",15:"4e9258",16:"b7ad59",17:"3bb9ff", 18:"d4a017",
    19:"f52887",20:"3b9c99" 
    }
rrd_periods = {
    'day':'1d', '3days':'3d', 'week':'1w', '2weeks':'2w', 'month':'1mon',
    '3months':'3mon', 'year':'1y'}

proc_types = ['short', 'medium', 'long', 'groups', 'other']
ds_proc_types = {
    'short':'Nshort',
    'medium':'Mmedium',
    'long':'Nlong',
    'groups':'Ngroups',
    'other':'Nother'
    }

stats = {1:'load_average', 2:'cpu', 3:'netxfer', 4:'memory'}

statLabels = {'load_average':'Load averages',
              'cpu':'CPU usage',
              'netxfer':'Network throughput',
              'memory':'Memory usage'}
ds = {
    'load_average':['la1', 'la5', 'la15'],
    'cpu':['user_cpu0', 'nice_cpu0', 'system_cpu0'],
    'netxfer':['rxd', 'txd'],
    'memory':['mem_ratio', 'swap']
      }
cdef = {
    'la1':'',
    'la5':'',
    'la15':'',
    'user_cpu0':'',
    'nice_cpu0':'',
    'system_cpu0':'',
    'rxd':'',
    'txd':'',
    'mem_ratio':' CDEF:cdef_mymem_ratio=mymem_ratio,100,* ',
    'swap':' CDEF:cdef_myswap=myswap,100,* '
    }
statSpecRRDStr = {
    'load_average':' --vertical-label="load avg" ',
    'cpu':' --upper-limit=100 --lower-limit=0.01 --rigid --vertical-label="% used" ',
    'netxfer':' --vertical-label="bytes/sec" ',
    'memory':' --upper-limit=100 --lower-limit=0.01 --rigid --vertical-label="% used" '
    }
statSpecTitleStr = {
    'load_average':' --title="Load Average for "',
    'cpu':' --title="CPU usage for "',
    'netxfer':' --title="Network throughput for "',
    'memory':' --title="Memory usage for "'
    }
dsSpecLegendStr = {
    'la1':'"Averaged over 1 min"',
    'la5':'"Averaged over 5 min"',
    'la15':'"Averaged over 15 min"',
    'user_cpu0':'"User"',
    'nice_cpu0':'"Nice"',
    'system_cpu0':'"System"',
    'rxd':'"Received"',
    'txd':'"Transmitted"',
    'mem_ratio':'"Real"',
    'swap':'"Swap"'
    }
graphdir = '/var/www/cgi-bin'
rrdtool_path = '/home/cafmon/bin'
rrd_path = '/home/cafmon/cafmon/RRD/'
genericStr = ''
MyURL = 'http://cdfcaf.fnal.gov/cgi-bin/caf/admin/graphs.py'
Style = """
<STYLE TYPE="text/css">
<!-- /* Hide from browsers that do not understand CSS */
BODY {
  font-family : Tahoma;
  color : black;
}
TD {
  font-family : Tahoma, Verdana, Arial;
  font-size: 11px;
  color : black;
}
TH {
  font-family : Verdana, Tahoma;
  font-size: 10px;
  font-weight: bold;
  color : black;
}
-->
</STYLE></head>
"""



# Method definition - numSecsInPeriod
#

def numSecsInPeriod(period):

    secsInDay = 60*60*24
    secsInWeek = 7*secsInDay
    secsInMonth = 30*secsInDay
    if period == "1day" or period == "1d":
        return secsInDay
    elif period == "3days" or period == "3d":
        return 3*secsInDay
    elif period == "1week" or period == "1w":
        return secsInWeek
    elif period == "2weeks" or period == "2w":
        return 2*secsInWeek
    elif period == "1month" or period == "1mon":
        return secsInMonth
    elif period == "3months" or period == "3mon":
        return 3*secsInMonth
    elif period == "1year" or period == "1y":
        return 365*secsInDay


# Method definition - color
#

def color(i):
    step = 51
    random.seed(i*10000000)
    R = step * ( int(random.random()*6.) )
    G = step * ( int(random.random()*6.) )
    B = step * ( int(random.random()*6.) )
    return "%02X%02X%02X" % (R,G,B)


def getActiveUsers(period="1day"):

    activeRunningList = []

    fileList = os.listdir(RRD_ROOT_DIR)
    for file in fileList:
        if file[:5] == "user_" and file[-4:] == ".rrd":

            rrdFile = os.path.join(RRD_ROOT_DIR, file)
            mtimeDiff = time.time() - os.path.getmtime(rrdFile)
            if mtimeDiff < numSecsInPeriod(period):

                tmpList = string.split(file, ".")
                i = string.find(file, tmpList[len(tmpList)-1])
                baseName = file[:i-1]
                userName = baseName[5:]

                command = "%s fetch %s AVERAGE --start -%s" % (RRDTOOL_BIN, rrdFile, period)
                childout, childin, childerr = popen2.popen3(command)
                childin.close()
                tempOut = childout.readlines()
                childout.close()
                tempErr = childerr.readlines()
                childerr.close()

                for line in tempErr:
                    line = string.strip(line)
                    print line

                activeRunning = 0
                for line in tempOut:
                    line = string.strip(line)
                    if string.find(line, ":") != -1:
                        dataTime, dataStr = string.split(line, ":")
                        pending, running, wait_time, start_time_diff, exec_time = string.split(dataStr)
                        if running != "nan":
                            activeRunning = 1
                if activeRunning:
                    activeRunningList.append(userName)

    return activeRunningList



# Head of HTML page
def htmlHead(title='CDF CAF Admin Monitoring Graphs', refresh=None):
    print 'Content-type: text/html\nCache-Control: No-cache\n\n'
    print '<html><head>'
    print '<title>%s</title>' % title
    if refresh != None:
        print '<meta http-equiv="refresh" content="%d">' % refresh
    print '</head>'
    print '<body bgcolor="#80DDCB" alink="#000000" vlink="#000000" link="#000000">'
    print '%s' % Style
    print
    print '  <p><table border=0>'
    print '    <tr valign="center">'
    print '    <td width="100"  valign="center" halign="center">'
    print '      <a href="http://cdfcaf.fnal.gov/">'
    print '      <img src="/caf/images/caf_logo.gif" border=0></a></td>'
    print '    <td valign="center" halign="center"><h1><font size=+10>'
    print '      CDF Central Analysis Facility<br></font></h1>'
    print '      <center><h1><font size=+5 color="#AA5500">Admin Monitoring Page</font><h1></center><br></td>'
    print '    </tr>'
    print '  </table>'
    print    
    print '  <hr>'
    print

# Tail of HTML page
def htmlTail():
    print '<hr><font size="-2"><center>'
    print '<a href="http://www-isd.fnal.gov/fcs/">FCS Group</a> | <a href="http://www-isd.fnal.gov/fbsng/">FBSNG</a>'
    print '</center>'
    print '</body></html>'

# Body of HTML page
def htmlBody():

    # Set default options
    logplot = 0
    period = 'day'

    # Store parameter strings pass to CGI script
    queue = cgi.FieldStorage()

    # Loop over CGI parameters and extract information
    nodeNames = []
    statNames = []
    allstat, allutil, allsrv, dCachesrv, \
             wrk, testCAFwrk, condorCAFwrk, dcachetestsrv, DBsrv \
             = (0, 0, 0, 0, 0, 0, 0, 0, 0)
    cgi_nodeStr = ''
    cgi_statStr = ''
    cgi_optStr = ''
    showBatchPlots = 1
    showAggAvg = 1
    for name in queue.keys():
        if name == 'period':
            period = queue[name].value
        elif(string.find(name,"node") > -1):
            if queue[name].value == "all":
                if name == "nodeutil":
                    allutil = 1
                if name == "nodesrv":
                    allsrv = 1
            elif (queue[name].value == "db" and name == "nodeutil"):
                DBsrv = 1
            elif (queue[name].value == "prod" and name == "node"):
                wrk = 1
            elif (queue[name].value == "test" and name == "node"):
                testCAFwrk = 1
            elif (queue[name].value == "condor" and name == "node"):
                condorCAFwrk = 1
            elif (queue[name].value == "dCache" and name == "nodesrv"):
                dCachesrv = 1
            elif (queue[name].value == "test" and name == "nodesrv"):
                dcachetestsrv = 1
            else:
                nodeNames.append(queue[name].value)
            cgi_nodeStr = cgi_nodeStr + name+'='+queue[name].value+'&'
        elif(string.find(name,"stat") > -1):
            if queue[name].value == "all":
                allstat = 1
            statNames.append(queue[name].value)
            cgi_statStr = cgi_statStr + name+'='+queue[name].value+'&'
        elif name == 'optlog':
            logplot = 1
            cgi_optStr = cgi_optStr + name+'='+queue[name].value+'&'
        elif name == 'nobatch':
            showBatchPlots = 0
            cgi_optStr = cgi_optStr + name+'='+queue[name].value+'&'
        elif name == 'noAggAvg':
            showAggAvg = 0 
            cgi_optStr = cgi_optStr + name+'='+queue[name].value+'&'

    nodeNames.sort()

    if allstat:
        statNames = stats.values()
    if allutil:
        nodeNames = nodeNames +  utility_nodes
    if allsrv:
        nodeNames = nodeNames + servers
    if wrk:
        nodeNames = nodeNames + worker_nodes
    if DBsrv:
        nodeNames = nodeNames + database_servers
    if testCAFwrk:
        nodeNames = nodeNames + testCAF_worker_nodes
    if condorCAFwrk:
	nodeNames = nodeNames + condorCAF_worker_nodes
    if dCachesrv:
        nodeNames = nodeNames + dCache_servers
    if dcachetestsrv:
        nodeNames = nodeNames + dcache_teststand_servers

    # Set up border around graphs
    print '<table width="100%" cellpadding="1" bgcolor="#103D3B" border=0 cellspacing=5>'
    print '  <tr><td>'
    print '  <table cellpadding=3 cellspacing=3><tr>'
    print '    <td bgcolor="#80DDCB">Period:</td>'
    for x in ['day','3days','week','2weeks','month','3months','year']:
        print '    <td bgcolor="#80DDCB">',
        if x == period:
            print '<b>%s</b>' % period,
        else:
            print '<a href="%s?%s%s%s&period=%s">%s</a>' %\
                  (MyURL, cgi_nodeStr, cgi_statStr, cgi_optStr, x, x),
        print '</td>'
    print '  </tr></table>'
    print '  </td></tr>'
    print '</table>'
    print
    print '<table  width="100%" border=0 bgcolor="#103D3B" cellspacing=5 cellpadding="1">'
    print '  <tr valign="top">'
    print '    <td><table  width="100%" bgcolor="#103D3B" border=0 cellspacing=5 cellpadding="1">'
    print '      <tr><td bgcolor="#80DDCB"><a href="http://cdfcaf.fnal.gov/cgi-bin/caf/admin/cafmon">Top Level</a></td></tr>'
    print '    </table>'

    # Assign starting period for graphs
    start = rrd_periods[period]

    print '    <td bgcolor="#80DDCB"><table width="100%" border=0>'
    # Display CAF usage graph
    if showBatchPlots:
        graph = '/caf/admin/graphs/farmUsage'+'_'+period+'.gif'
        graphGlobal = " -h 150 --c BACK\#000000 -c CANVAS\#111133 -c MGRID\#CCCCCC -c FONT\#00EEBB"
        farmUsageStr = " --title=\"Number of Running Processes\" --lower-limit=0. --upper-limit=900. --rigid DEF:val1=/home/cafmon/batchmon/RRD/running.rrd:Nshort:AVERAGE LINE2:val1#FFFF00:short DEF:val2=/home/cafmon/batchmon/RRD/running.rrd:Nmedium:AVERAGE LINE2:val2#BB0000:medium DEF:val3=/home/cafmon/batchmon/RRD/running.rrd:Nlong:AVERAGE LINE2:val3#3333FF:long DEF:val4=/home/cafmon/batchmon/RRD/running.rrd:Ngroups:AVERAGE LINE2:val4#00FF00:groups DEF:val5=/home/cafmon/batchmon/RRD/running.rrd:Nother:AVERAGE LINE2:val5#FF00FF:others CDEF:tot=val1,val2,val3,val4,val5,+,+,+,+ LINE2:tot#aaaaaa:total"
        str = rrdtool_path+'/rrdtool graph '+graphdir+graph+\
            ' --start -'+start+graphGlobal+farmUsageStr
        commands.getoutput(str)
        print '      <tr><td><img src="http://cdfcaf.fnal.gov/cgi-bin/caf/admin/getgraph.py?graph=farmUsage'+'_'+period+'.gif"></tr></td>'
        print '<tr><td><hr></tr></td>'


    # Display CAF user usage graph

    if showBatchPlots:
        graphFile = "/var/www/cgi-bin/caf/admin/graphs/activeUsers_running_%s.gif" % period
        graphStr = "-h 250 -c BACK\#000000 -c CANVAS\#111133 -c FONT\#00EEBB -c GRID\#000000 --title=\"Number of Running Processes\" --lower-limit=0 --start -"+start

        activeUserList = getActiveUsers(start)

        i = 0
        for userName in activeUserList:
            i = i + 1
            colorStr = "#%s" % color(i)
            graphStr = graphStr + " DEF:val%d=%s/user_%s.rrd:Nrunning:AVERAGE CDEF:nval%d=val%d,UN,0,val%d,IF LINE2:nval%d%s:%s" % (i, RRD_ROOT_DIR, userName, i, i, i, i, colorStr, userName)
        print '<br>'
        command = "%s graph %s %s" % (RRDTOOL_BIN, graphFile, graphStr)
        commands.getoutput(command)
        print '      <tr><td><img src="http://cdfcaf.fnal.gov/cgi-bin/caf/admin/getgraph.py?graph=activeUsers_running_'+period+'.gif"></tr></td>'
        print '<tr><td><hr></tr></td>'


    # Print averages for all requested nodes
    print '<tr><td><a NAME="avg"><h2>Averages</h2></a></tr></td>'

    if showAggAvg:
        for statName in statNames:
            graph = '/caf/admin/graphs/avg_'+statName+'_'+period+\
                    '.gif'
            statSpecStr = statSpecRRDStr[statName]
            titleStr = statSpecTitleStr[statName]
            dsList = ds[statName]
            dsStr = ''
            cdefStr = ''
            color_idx = 0
            for dsName in dsList:
                color_idx = color_idx + 1
                colorStr = '#'+colors[color_idx]
                legendStr = dsSpecLegendStr[dsName]
                idx = 0
                for nodeName in nodeNames:
                    sidx = "_%d" % idx
                    dsStr = dsStr + 'DEF:my'+dsName+sidx+'='+rrd_path+'/'+\
                            nodeName+'.rrd:'+dsName+':AVERAGE '
                    idx = idx + 1

                cdefStr = cdefStr + 'CDEF:'+dsName+'_sum1=0,my'+dsName+'_0,+ '
                for n in range(2, (len(nodeNames)+1)):
                    sn = "%d" % n
                    snm1 = "%d" % (n-1)
                    cdefStr = cdefStr + 'CDEF:'+dsName+'_sum'+sn+'='+\
                              dsName+'_sum'+snm1+',my'+dsName+'_'+snm1+',+ '
                snumnodes = "%d" % len(nodeNames)

                if len(cdef[dsName]):
                    cdefStr = cdefStr + 'CDEF:'+dsName+'_avg='+dsName+'_sum'+\
                              snumnodes+','+snumnodes+',/ '+\
                              'CDEF:cdef_'+dsName+'_avg='+dsName+'_avg,100,* '+\
                              'LINE2:cdef_'+dsName+'_avg' +\
                              colorStr+':'+legendStr+' '
                else:
                    cdefStr = cdefStr + 'CDEF:'+dsName+'_avg='+dsName+'_sum'+\
                              snumnodes+','+snumnodes+',/ LINE2:'+dsName+'_avg' +\
                              colorStr+':'+legendStr+' '
                
            str = rrdtool_path+'/rrdtool graph '+graphdir+graph+ ' --start -'+\
                  start+titleStr+'all '+genericStr+statSpecStr+dsStr+cdefStr
        
            commands.getoutput(str)
            print '      <tr><td><img src="http://cdfcaf.fnal.gov/cgi-bin/caf/admin/getgraph.py?graph=avg_'+statName+'_'+period+'.gif"></tr></td>'
        print '<tr><td><hr></tr></td>'


# Show aggregate for network bandwidth

        print '<tr><td><a NAME="Aggregate"><h2>Aggregate</h2></a></tr></td>'
        for statName in statNames:
            if statName == 'netxfer':
                graph = '/caf/admin/graphs/agg_'+statName+'_'+period+\
                        '.gif'
                statSpecStr = statSpecRRDStr[statName]
                titleStr = statSpecTitleStr[statName]
                dsList = ds[statName]
                dsStr = ''
                cdefStr = ''
                color_idx = 0
                for dsName in dsList:
                    color_idx = color_idx + 1
                    colorStr = '#'+colors[color_idx]
                    legendStr = dsSpecLegendStr[dsName]
                    idx = 0
                    for nodeName in nodeNames:
                        sidx = "%d" % idx
                        dsStr = dsStr + 'DEF:my'+dsName+sidx+'='+rrd_path+'/'+\
                                nodeName+'.rrd:'+dsName+':AVERAGE '
                        idx = idx + 1

                    cdefStr = cdefStr + 'CDEF:'+dsName+'_sum1=0,my'+dsName+'0,+ '
                    for n in range(2, (len(nodeNames)+1)):
                        sn = "%d" % n
                        snm1 = "%d" % (n-1)
                        cdefStr = cdefStr + 'CDEF:'+dsName+'_sum'+sn+'='+\
                                  dsName+'_sum'+snm1+',my'+dsName+snm1+',+ '
                    snumnodes = "%d" % len(nodeNames)

                    cdefStr = cdefStr + 'LINE2:'+dsName+'_sum'+snumnodes+\
                              colorStr+':'+legendStr+' '
                
                str = rrdtool_path+'/rrdtool graph '+graphdir+graph+ ' --start -'+\
                      start+titleStr+'all '+genericStr+statSpecStr+dsStr+cdefStr
        
                commands.getoutput(str)
                print '      <tr><td><img src="http://cdfcaf.fnal.gov/cgi-bin/caf/admin/getgraph.py?graph=agg_'+statName+'_'+period+'.gif"></tr></td>'
                print '<tr><td><hr></tr></td>'

    print '<tr><td>Go to statistic:<br>'
    for x in statLabels.values():
        print '<a HREF="#'+x+'">'+x+'</a>'
    print '<br><br></tr></td>'

    # Loop over all requested nodes and statistics to graph
    for statName in statNames:
        print '<tr><td><a NAME="'+statLabels[statName]+'"><h2>'+statLabels[statName]+'</h2></a></tr></td>'
        for nodeName in nodeNames:
            graph = '/caf/admin/graphs/'+nodeName+'_'+statName+'_'+period+\
                    '.gif'
            statSpecStr = statSpecRRDStr[statName]
            if logplot:
                statSpecStr = statSpecStr+'--logarithmic '
            titleStr = statSpecTitleStr[statName]
            dsList = ds[statName]
            dsStr = ''
            color_idx = 0
            for dsName in dsList:
                color_idx = color_idx + 1
                colorStr = '#'+colors[color_idx]
                legendStr = dsSpecLegendStr[dsName]
                cdefStr = cdef[dsName]
                if len(cdefStr):
                    dsStr = dsStr + 'DEF:my'+dsName+'='+rrd_path+'/'+nodeName+\
                            '.rrd:'+dsName+':AVERAGE '+cdefStr+\
                            'LINE2:cdef_my'+dsName+colorStr+':'+legendStr+' '
                else:
                    dsStr = dsStr + 'DEF:my'+dsName+'='+rrd_path+'/'+nodeName+\
                            '.rrd:'+dsName+':AVERAGE LINE2:my'+dsName+\
                            colorStr+':'+legendStr+' '
                
            str = rrdtool_path+'/rrdtool graph '+graphdir+graph+ ' --start -'+\
                  start+titleStr+nodeName+genericStr+statSpecStr+dsStr
#            print str
            commands.getoutput(str)
            print '      <tr><td><img src="http://cdfcaf.fnal.gov/cgi-bin/caf/admin/getgraph.py?graph='+nodeName+'_'+statName+'_'+period+'.gif"></tr></td>'
        print '<tr><td><hr></tr></td>'

    print '    </td></table>'
    print '  </td></tr>'
    print '</table>'

# Main program
if __name__ == '__main__':
    htmlHead()
    htmlBody()
    htmlTail()

